from Ft.Xml import Parse, CreateInputSource
from Ft.Xml.Domlette import Print, EntityReader, GetAllNs, ParseFragment
from Ft.Lib import Uri, Uuid
from xml.dom import Node
import cStringIO, os

SOURCE1 = "<p>hello<b>world</b>.  How ya doin'?</p>"

EXPECTED1 = '<p>hello<b>world</b>.  How ya doin\'?</p>'

EXPECTED2 = '<p xmlns="http://www.w3.org/1999/xhtml">hello<b>world</b>.  How ya doin\'?</p>'

SOURCE2 = "<p>hello<b xmlns=''>world</b>.  How ya doin'?</p>"

EXPECTED3 = '<p xmlns="http://www.w3.org/1999/xhtml">hello<b xmlns="">world</b>.  How ya doin\'?</p>'

SOURCE3 = "<h:p xmlns:h='http://www.w3.org/1999/xhtml'>hello<h:b>world</h:b>.  How ya doin'?</h:p>"

EXPECTED4 = '<h:p xmlns:h="http://www.w3.org/1999/xhtml">hello<h:b>world</h:b>.  How ya doin\'?</h:p>'


def Test(tester):
    tester.startGroup("Fragment parse")

    tester.startTest("Plain parse")
    isrc = CreateInputSource(SOURCE1)
    doc = ParseFragment(isrc)
    stream = cStringIO.StringIO()
    Print(doc, stream)
    tester.compare(EXPECTED1, stream.getvalue())
    #Minimal node testing
    tester.compare(len(doc.childNodes), 1)
    tester.compare(doc.childNodes[0].nodeType, Node.ELEMENT_NODE)
    tester.compare(doc.childNodes[0].nodeName, u'p')
    tester.compare(doc.childNodes[0].namespaceURI, None)
    tester.compare(doc.childNodes[0].prefix, None,)
    tester.testDone()

    tester.startTest("Parse with overridden default namespace")
    nss = {u'xml': u'http://www.w3.org/XML/1998/namespace',
           None: u'http://www.w3.org/1999/xhtml'}
    isrc = CreateInputSource(SOURCE1)
    doc = ParseFragment(isrc, nss)
    stream = cStringIO.StringIO()
    Print(doc, stream)
    tester.compare(EXPECTED2, stream.getvalue())
    #doc = ParseFragment(TEST_STRING)
    #Minimal node testing
    tester.compare(len(doc.childNodes), 1)
    tester.compare(doc.childNodes[0].nodeType, Node.ELEMENT_NODE)
    tester.compare(doc.childNodes[0].nodeName, u'p')
    tester.compare(doc.childNodes[0].namespaceURI, u'http://www.w3.org/1999/xhtml')
    tester.compare(doc.childNodes[0].prefix, None,)
    tester.testDone()

    tester.startTest("Parse with overridden default namespace and re-overridden child")
    nss = {u'xml': u'http://www.w3.org/XML/1998/namespace',
           None: u'http://www.w3.org/1999/xhtml'}
    isrc = CreateInputSource(SOURCE2)
    doc = ParseFragment(isrc, nss)
    stream = cStringIO.StringIO()
    Print(doc, stream)
    tester.compare(EXPECTED3, stream.getvalue())
    #Minimal node testing
    tester.compare(len(doc.childNodes), 1)
    tester.compare(doc.childNodes[0].nodeType, Node.ELEMENT_NODE)
    tester.compare(doc.childNodes[0].nodeName, u'p')
    tester.compare(doc.childNodes[0].namespaceURI, u'http://www.w3.org/1999/xhtml')
    tester.compare(doc.childNodes[0].prefix, None,)
    tester.testDone()

    tester.startTest("Parse with overridden non-default namespace")
    nss = {u'xml': u'http://www.w3.org/XML/1998/namespace',
           u'h': u'http://www.w3.org/1999/xhtml'}
    isrc = CreateInputSource(SOURCE3)
    doc = ParseFragment(isrc, nss)
    stream = cStringIO.StringIO()
    Print(doc, stream)
    tester.compare(EXPECTED4, stream.getvalue())
    #doc = ParseFragment(TEST_STRING)
    #Minimal node testing
    tester.compare(len(doc.childNodes), 1)
    tester.compare(doc.childNodes[0].nodeType, Node.ELEMENT_NODE)
    tester.compare(doc.childNodes[0].nodeName, u'h:p')
    tester.compare(doc.childNodes[0].namespaceURI, u'http://www.w3.org/1999/xhtml')
    tester.compare(doc.childNodes[0].prefix, u'h')
    tester.testDone()

    tester.groupDone()
    return




########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/__init__.py,v 1.49.4.3 2006/09/18 13:49:48 uogbuji Exp $
"""
Module providing utilities for processing XML files

Copyright 2006 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__all__ = [# constants
           'EMPTY_NAMESPACE',
           'EMPTY_PREFIX',
           'XML_NAMESPACE',
           'XMLNS_NAMESPACE',
           'XHTML_NAMESPACE',
           'READ_EXTERNAL_DTD',
           'HAS_PYEXPAT',
           # classes
           'ReaderException', 'XIncludeException',
           # functions
           'SplitQName', 'ApplyXUpdate',
           # modules
           'MarkupWriter',
           'Parse',
           'ParsePath',
           ]


EMPTY_NAMESPACE = None
EMPTY_PREFIX = None
XML_NAMESPACE = u"http://www.w3.org/XML/1998/namespace"
XMLNS_NAMESPACE = u"http://www.w3.org/2000/xmlns/"
XHTML_NAMESPACE = u"http://www.w3.org/1999/xhtml"

# Defined here to ensure that all readers operate the same.
READ_EXTERNAL_DTD = True

from Ft import FtException, __version__, TranslateMessage as _

class ReaderException(FtException):
    """
    Exception class for errors specific to XML reading
    (at a level above standard, non-namespace-aware parsing)
    """
    # Fatal errors
    # Note: These are actual Expat error codes redefined here to allow for
    #   translation of the error messages.
    #NO_MEMORY = 1                              # mapped to MemoryError
    SYNTAX_ERROR = 2
    NO_ELEMENTS = 3
    INVALID_TOKEN = 4
    UNCLOSED_TOKEN = 5
    PARTIAL_CHAR = 6
    TAG_MISMATCH = 7
    DUPLICATE_ATTRIBUTE = 8
    JUNK_AFTER_DOCUMENT_ELEMENT = 9
    ILLEGAL_PARAM_ENTITY_REF = 10
    UNDEFINED_ENTITY = 11
    RECURSIVE_ENTITY_REF = 12
    ASYNC_ENTITY = 13
    BAD_CHAR_REF = 14
    BINARY_ENTITY_REF = 15
    ATTRIBUTE_EXTERNAL_ENTITY_REF = 16
    MISPLACED_XML_PI = 17
    UNKNOWN_ENCODING = 18
    INCORRECT_ENCODING = 19
    UNCLOSED_CDATA_SECTION = 20
    EXTERNAL_ENTITY_HANDLING = 21
    NOT_STANDALONE = 22
    #UNEXPECTED_STATE = 23                      # mapped to SystemError
    ENTITY_DECLARED_IN_PE = 24
    #FEATURE_REQUIRES_XML_DTD = 25              # mapped to SystemError
    #CANT_CHANGE_FEATURE_ONCE_PARSING = 26      # mapped to SystemError
    UNBOUND_PREFIX = 27
    UNDECLARED_PREFIX = 28
    INCOMPLETE_PE = 29
    INVALID_XML_DECL = 30
    INVALID_TEXT_DECL = 31
    INVALID_PUBLICID = 32
    #SUSPENDED = 33                             # mapped to SystemError
    #NOT_SUSPENDED = 34                         # mapped to RuntimeError
    #ABORTED = 35                               # mapped to SystemError
    #FINISHED = 36                              # mapped to SystemError
    #SUSPEND_PE = 37                            # mapped to SystemError
    RESERVED_PREFIX_XML = 38
    RESERVED_PREFIX_XMLNS = 39
    RESERVED_NAMESPACE_URI = 40

    # Validity errors
    MISSING_DOCTYPE = 1000
    INVALID_ELEMENT = 1001
    ROOT_ELEMENT_MISMATCH = 1002
    UNDECLARED_ELEMENT = 1003
    INCOMPLETE_ELEMENT = 1004
    INVALID_TEXT = 1005
    UNDECLARED_ATTRIBUTE = 1006
    DUPLICATE_ID = 1007
    UNDECLARED_ENTITY = 1008
    INVALID_ENTITY = 1009
    UNDECLARED_NOTATION = 1010
    MISSING_ATTRIBUTE = 1011
    UNDEFINED_ID = 1012                         # FIXME: implement
    DUPLICATE_ELEMENT_DECL = 1013
    DUPLICATE_ID_DECL = 1014
    ID_ATTRIBUTE_DEFAULT = 1015
    XML_SPACE_DECL = 1016
    XML_SPACE_VALUES = 1017
    INVALID_NAME_VALUE = 1018
    INVALID_NAME_SEQ_VALUE = 1019
    INVALID_NMTOKEN_VALUE = 1020
    INVALID_NMTOKEN_SEQ_VALUE = 1021
    INVALID_ENUM_VALUE = 1022
    ATTRIBUTE_UNDECLARED_NOTATION = 1023
    ENTITY_UNDECLARED_NOTATION = 1024           # FIXME: implement

    # Warnings
    ATTRIBUTES_WITHOUT_ELEMENT = 2000
    ATTRIBUTE_DECLARED = 2001
    ENTITY_DECLARED = 2002

    XML_PARSE_ERROR = 100
    RECURSIVE_PARSE_ERROR = 101

    def __init__(self, errorCode, systemId, lineNumber, columnNumber,
                 **kwords):
        FtException.__init__(self, errorCode, MessageSource.READER, **kwords)
        self.systemId = systemId
        self.lineNumber = lineNumber
        self.columnNumber = columnNumber
        return

    def __str__(self):
        systemId = self.systemId
        if isinstance(systemId, unicode):
            systemId = systemId.encode('unicode_escape')
        return _("In %s, line %s, column %s: %s") % (systemId,
                                                     self.lineNumber,
                                                     self.columnNumber,
                                                     self.message)

class XIncludeException(FtException):
    """
    Exception class for errors specific to XInclude processing
    """
    MISSING_HREF = 10
    INVALID_PARSE_ATTR = 11
    TEXT_XPOINTER = 12
    FRAGMENT_IDENTIFIER = 13
    UNSUPPORTED_XPOINTER = 14
    INCLUDE_IN_INCLUDE = 15
    FALLBACK_NOT_IN_INCLUDE = 16
    MULTIPLE_FALLBACKS = 17

    def __init__(self, errorCode, *args):
        FtException.__init__(self, errorCode, MessageSource.XINCLUDE, args)

import MessageSource

from Ft.Xml.Lib.XmlString import SplitQName


#Wrap this so that we can import it later
def ApplyXUpdate(*args, **kw_args):
    import Ft.Xml.XUpdate
    return XUpdate.ApplyXUpdate(*args, **kw_args)

#Good ol' backward compatibility for creative spellings
def ApplyXupdate(*args, **kw_args):
    import Ft.Xml.XUpdate
    return XUpdate.ApplyXupdate(*args, **kw_args)


from distutils import version
pyxml_required = version.StrictVersion('0.8.0')
def CheckVersion(feature=None):
    """
    PyXML is required by some features of 4Suite (e.g., validating parsing,
    and 4XSLT's SaxWriter). This is a common function to test whether a
    correct version of PyXML is installed. It raises a SystemExit if the
    test result is negative, and returns None otherwise.

    The feature argument is a string indicating which feature in 4Suite
    requires PyXML. It is output as part of the SystemExit message.
    """
    try:
        import _xmlplus
        xml_version = version.StrictVersion(_xmlplus.__version__)
    except:
        xml_version = version.StrictVersion('0.0.0')

    if xml_version < pyxml_required:
        import sys
        if not feature:
            feature = "this feature"
        print """
        PyXML v%s is required for %s in 4Suite.
        It is available at http://sourceforge.net/projects/pyxml.
        """ % (str(pyxml_required), feature)
        sys.exit(1)

HAS_PYEXPAT = True
try:
    from xml.parsers import expat
    expat.ParserCreate
    del expat
except (ImportError, AttributeError):
    HAS_PYEXPAT = False

from MarkupWriter import MarkupWriter

# Convenience functions for parsing, etc.
import os

def CreateInputSource(obj, uri=None):
    """
    Convenience function for creating an InputSource.
    obj - a string, Unicode object (only if you really know what you're doing),
          file-like object (stream), file path or URI.  You can also pass an
          InputSource object, in which case the return value is just the same
          object, possibly with the URI modified
    uri - optional override URI.  The base URI for the IS will be set to this
          value

    Returns an InputSource which can be passed to 4Suite APIs.
    """
    #do the imports within the function: a tad bit less efficient, but
    #avoid circular crap
    from Ft.Xml import InputSource
    factory = InputSource.DefaultFactory
    from Ft.Lib import Uri, Uuid
    from Ft.Xml.Lib.XmlString import IsXml

    if isinstance(obj, InputSource.InputSource):
        isrc = obj
    elif hasattr(obj, 'read'):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        isrc = factory.fromStream(obj, dummy_uri)
    elif IsXml(obj):
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        isrc = factory.fromString(obj, dummy_uri)
    elif Uri.IsAbsolute(obj): #or not os.path.isfile(obj):
        isrc =  factory.fromUri(obj)
    else:
        isrc = factory.fromUri(Uri.OsPathToUri(obj))
    if uri: isrc.uri = uri
    return isrc


def Parse(source):
    """
    Convenience function for parsing XML.  Use this function with a single
    argument, which must either be a string (not Unicode object), file-like
    object (stream), file path or URI.

    Returns a Domlette node.

    Only pass strings or streams to this function if the XML is self-contained
    XML (i.e. not requiring access to any other resource such as external
    entities or includes).  If you get URI resolution errors, do not use this
    function: use the lower-level APIs instead.  As an example, if you want
    such resolution to use the current working directory as a base, parse
    as follows for strings:

    from Ft.Xml.Domlette import NonvalidatingReader
    from Ft.Lib import Uri

    XML = "<!DOCTYPE a [ <!ENTITY b "b.xml"> ]><a>&b;</a>"

    base = Uri.OsPathToUri('')  #Turn CWD into a file: URL
    doc = NonvalidatingReader.parseString(XML, base)
    # during parsing, the replacement text for &b;
    # will be obtained from b.xml in the CWD

    For streams, use "parseStream" rather than "parseString" in the above.
    """
    #do the imports within the function: a tad bit less efficient, but
    #avoid circular crap
    from Ft.Xml.Domlette import NonvalidatingReader
    from Ft.Lib import Uri, Uuid
    from Ft.Xml.Lib.XmlString import IsXml

    if hasattr(source, 'read'):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        return NonvalidatingReader.parseStream(source, dummy_uri)
    elif IsXml(source):
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        return NonvalidatingReader.parseString(source, dummy_uri)
    elif Uri.IsAbsolute(source): #or not os.path.isfile(source):
        return NonvalidatingReader.parseUri(source)
    else:
        return NonvalidatingReader.parseUri(Uri.OsPathToUri(source))


def ParsePath(source):
    import warnings
    warnings.warn("You are using the deprecated Ft.Xml.ParsePath function, Please use Ft.Xml.Parse instead", DeprecationWarning, 2)

    return Parse(source)





import os
import sys
import time
from distutils import spawn, sysconfig, util
from distutils.ccompiler import new_compiler, show_compilers
from distutils.core import Command
from distutils.dep_util import newer_group

from Ft.Lib import ImportUtil
from Ft.Lib.DistExt import Util, ImageHlp
from Ft.Lib.DistExt.Structures import Script, Executable

SHELL_SCRIPT_BODY = """#!%(executable)s
# %(name)s script generated by %(command)s on %(timestamp)s.
# DO NOT EDIT THIS FILE!

import %(module)s
status = %(module)s.%(function)s()
raise SystemExit(status)
"""

class ScriptInfo(ImageHlp.Struct):
    """
    Representation of the SCRIPT_INFO resource in the stub executable.
    """
    __fields__ = [
        (ImageHlp.Dword, 'Signature'),
        (ImageHlp.Word, 'MajorPythonVersion'),
        (ImageHlp.Word, 'MinorPythonVersion'),
        (ImageHlp.Word, 'Subsystem'),
        (ImageHlp.Word, 'Characteristics'),
        (ImageHlp.Word, 'ScriptAddress'),
        (ImageHlp.Word, 'ScriptSize'),
        ]

class BuildScripts(Command):

    command_name = 'build_scripts'

    description = "\"build\" scripts"

    user_options = [
        ('build-dir=', 'd', "directory to \"build\" (copy) to"),
        ('build-temp=', 't',
         "directory for temporary files (build by-products)"),
        ('force', 'f', "forcibly build everything (ignore file timestamps"),
        ('debug', 'g', "compile/link with debugging information"),
        ('compiler=', 'c', "specify the compiler type"),
        ]

    help_options = [
        ('help-compiler', None,
         "list available compilers", show_compilers),
        ]

    boolean_options = ['force', 'debug']

    def initialize_options(self):
        self.build_dir = None
        self.build_temp = None
        self.force = None
        self.debug = None
        self.compiler = None
        return

    def finalize_options(self):
        undefined_temp = self.build_temp is None
        self.set_undefined_options('build',
                                   ('build_scripts', 'build_dir'),
                                   ('build_temp', 'build_temp'),
                                   ('compiler', 'compiler'),
                                   ('debug', 'debug'),
                                   ('force', 'force'))
        if undefined_temp:
            self.build_temp = os.path.join(self.build_temp, 'scripts')

        self.scripts = self.distribution.scripts or []

        # Get the linker arguments for building executables
        if os.name == 'posix':
            args = sysconfig.get_config_vars('LDFLAGS', 'LINKFORSHARED')
            self.link_preargs = ' '.join(args).split()

            args = sysconfig.get_config_vars('LIBS', 'MODLIBS', 'SYSLIBS',
                                             'LDLAST')
            self.link_postargs = ' '.join(args).split()
        else:
            self.link_preargs = []
            self.link_postargs = []

        # Get the extension for executables
        self.exe_extension = sysconfig.get_config_var('EXE') or ''
        if self.debug and os.name == 'nt':
            self.exe_extension = '_d' + self.exe_extension
        return

    def run(self):
        """
        Create the proper script for the current platform.
        """
        if not self.scripts:
            return

        # Ensure the destination directory exists.
        self.mkpath(self.build_dir)

        # Build the "plain" (pure-Python) scripts.
        self.build_scripts([ script for script in self.scripts
                             if isinstance(script, Script) ])

        # Build the executable (compiled) scripts.
        self.build_executables([ script for script in self.scripts
                                 if isinstance(script, Executable) ])
        return

    # -- worker functions ---------------------------------------------

    def build_scripts(self, scripts):
        for script in scripts:
            self.build_script(script)
        return

    def build_script(self, script):
        """
        Builds a CommandLineApp script.
        On POSIX systems, this is a generated shell script.  For Windows,
        it is a compiled executable with the generated file appended to the
        end of the stub.
        """
        # Get the destination filename
        outfile = self.get_script_filename(script)

        # Determine if the script needs to be built
        command_mtime = ImportUtil.GetLastModified(__name__)
        if os.name == 'nt':
            stub_mtime = ImportUtil.GetResourceLastModified(__name__,
                                                            'stubmain.exe')
            command_mtime = max(command_mtime, stub_mtime)
        try:
            target_mtime = os.stat(outfile).st_mtime
        except OSError:
            target_mtime = -1
        if not (self.force or command_mtime > target_mtime):
            self.announce("skipping '%s' script (up-to-date)" % script.name)
            return
        else:
            self.announce("building '%s' script" % (script.name), 2)

        repl = {'executable' : self.get_python_executable(),
                'command' : self.get_command_name(),
                'timestamp' : time.asctime(),
                'toplevel' : script.module.split('.', 1)[0],
                }
        repl.update(vars(script))
        script_body = SHELL_SCRIPT_BODY % repl

        if self.dry_run:
            # Don't actually create the script
            pass
        elif os.name == 'nt':
            # Populate the ScriptInfo structure
            script_info = ScriptInfo()
            script_info.Signature = 0x00005446  # "FT\0\0"
            script_info.MajorPythonVersion = sys.version_info[0]
            script_info.MinorPythonVersion = sys.version_info[1]
            script_info.Subsystem = 0x0003;     # CUI
            if self.debug:
                script_info.Characteristics |= 0x0001
            stub_bytes = ImportUtil.GetResourceString(__name__, 'stubmain.exe')
            script_info.ScriptAddress = len(stub_bytes)
            script_info.ScriptSize = len(script_body)

            # Write the script executable
            f = open(outfile, 'w+b')
            try:
                f.write(stub_bytes)
                f.write(script_body)
                ImageHlp.UpdateResource(f, ImageHlp.RT_RCDATA, 1, script_info)
                ImageHlp.SetSubsystem(f, ImageHlp.IMAGE_SUBSYSTEM_WINDOWS_CUI)
            finally:
                f.close()
        else:
            # Create the file with execute permissions set
            fd = os.open(outfile, os.O_WRONLY|os.O_CREAT|os.O_TRUNC, 0755)
            try:
                os.write(fd, script_body)
            finally:
                os.close(fd)
        return

    def build_executables(self, executables):
        if not executables:
            return

        # Create the compiler for compiling the executables.
        self._prep_compiler()

        for executable in executables:
            self.build_executable(executable)
        return

    def build_executable(self, executable):
        """
        Builds a compiled executable.
        For all systems, the executable is created in the same fashion as
        the Python interpreter executable.
        """
        outfile = self.get_script_filename(executable)

        all_sources = self._prep_build(script)
        sources = []
        for source, includes in all_sources:
            sources.append(source)
            sources.extend(includes)

        if not (self.force or newer_group(sources, outfile, 'newer')):
            self.announce("skipping '%s' executable (up-to-date)" %
                          executable.name)
            return
        else:
            self.announce("building '%s' executable" % executable.name)

        output_dir = os.path.join(self.build_temp, executable.name)

        macros = executable.define_macros[:]
        for undef in executable.undef_macros:
            macros.append((undef,))

        objects = []
        for source, includes in all_sources:
            if not self.force:
                # Recompile if the includes or source are newer than the
                # resulting object files.
                objs = self.compiler.object_filenames([source], 1, output_dir)

                # Recompile if any of the inputs are newer than the object
                inputs = [source] + includes
                force = 0
                for filename in objs:
                    force = force or newer_group(inputs, filename, 'newer')
                self.compiler.force = force

            objs = self.compiler.compile(
                [source],
                output_dir=output_dir,
                macros=macros,
                include_dirs=executable.include_dirs,
                debug=self.debug,
                extra_postargs=executable.extra_compile_args)
            objects.extend(objs)

        # Reset the force flag on the compiler
        self.compiler.force = self.force

        # Now link the object files together into a "shared object" --
        # of course, first we have to figure out all the other things
        # that go into the mix.
        if os.name == 'nt' and self.debug:
            executable = executable.name + '_d'
        else:
            executable = executable.name

        if executable.extra_objects:
            objects.extend(executable.extra_objects)

        # On Windows, non-MSVC compilers need some help finding python
        # libs. This logic comes from distutils/command/build_ext.py.
        libraries = executable.libraries
        if sys.platform == "win32":
            from distutils.msvccompiler import MSVCCompiler
            if not isinstance(self.compiler, MSVCCompiler):
                template = "python%d%d"
                if self.debug:
                    template = template + "_d"
                pythonlib = (template % ((sys.hexversion >> 24),
                                         (sys.hexversion >> 16) & 0xff))
                libraries += [pythonlib]

        self.compiler.link_executable(
            objects, executable,
            libraries=libraries,
            library_dirs=executable.library_dirs,
            runtime_library_dirs=executable.runtime_library_dirs,
            extra_preargs=self.link_preargs,
            extra_postargs=self.link_postargs + executable.extra_link_args,
            debug=self.debug,
            build_temp=self.build_temp)
        return

    # -- utility functions --------------------------------------------

    def get_python_executable(self):
        if os.name == 'nt':
            executable = sys.executable
        else:
            executable = spawn.find_executable('env')
            if executable is None:
                # No 'env' executable found; use the interpreter directly
                executable = sys.executable
            else:
                # Use the python found runtime (via env)
                executable += ' python'
        return executable

    def get_script_filename(self, script):
        """
        Convert the name of a script into the name of the file which it
        will be run from.
        """
        # All Windows scripts are executables
        if os.name == 'nt' or isinstance(script, Executable):
            script_name = script.name + self.exe_extension
        else:
            script_name = script.name
        return os.path.join(self.build_dir, script_name)

    # -- helper functions ---------------------------------------------

    def _prep_compiler(self):
        # Setup the CCompiler object that we'll use to do all the
        # compiling and linking
        self.compiler = new_compiler(compiler=self.compiler,
                                     verbose=self.verbose,
                                     dry_run=self.dry_run,
                                     force=self.force)
        sysconfig.customize_compiler(self.compiler)

        # If we were asked to build any C/C++ libraries, make sure that the
        # directory where we put them is in the library search path for
        # linking executables.
        if self.distribution.has_c_libraries():
            build_clib = self.get_finalized_command('build_clib')
            self.compiler.set_libraries(build_clib.get_library_names())
            self.compiler.add_library_dir(build_clib.build_clib)

        # Make sure Python's include directories (for Python.h, pyconfig.h,
        # etc.) are in the include search path.
        py_include = sysconfig.get_python_inc()
        plat_py_include = sysconfig.get_python_inc(plat_specific=1)
        self.compiler.add_include_dir(py_include)
        if plat_py_include != py_include:
            include_dirs.append(plat_py_include)

        if os.name == 'posix':
            # Add the Python archive library
            ldlibrary = sysconfig.get_config_var('BLDLIBRARY')
            # MacOSX with frameworks doesn't link against a library
            if ldlibrary:
                # Get the location of the library file
                for d in sysconfig.get_config_vars('LIBDIR', 'LIBP', 'LIBPL'):
                    library = os.path.join(d, ldlibrary)
                    if os.path.exists(library):
                        self.compiler.add_link_object(library)
                        break
        elif os.name == 'nt':
            # Add Python's library directory
            lib_dir = os.path.join(sys.exec_prefix, 'libs')
            self.compiler.add_library_dir(lib_dir)
        return

    def _prep_build(self, script):
        # This should really exist in the CCompiler class, but
        # that would required overriding all compilers.
        result = []
        for source in script.sources:
            source = util.convert_path(source)
            includes = Util.FindIncludes(source, script.include_dirs)
            result.append((source, includes))
        return result

    # -- external interfaces ------------------------------------------

    def get_outputs(self):
        return [ self.get_script_filename(script) for script in self.scripts ]

    def get_source_files(self):
        filenames = []
        for script in self.scripts:
            if isinstance(script, Executable):
                for source, includes in self._prep_build(script):
                    filenames.append(source)
                    filenames.extend(includes)
        return filenames



import struct

__all__ = ['DataType', 'Array', 'Struct', 'Byte', 'Word', 'Dword', 'ArrayOf',
           'RT_RCDATA', 'RT_VERSION', 'UpdateResource', 'SetSubsystem',
           ]

try:
    # Python 2.3+
    issubclass(type, ())
except TypeError:
    # Python 2.2
    import __builtin__
    def issubclass(subclass, class_or_tuple):
        if isinstance(class_or_tuple, tuple):
            for class_ in class_or_tuple:
                if issubclass(subclass, class_):
                    return True
            return False
        return __builtin__.issubclass(subclass, class_or_tuple)

class DataType(type):
    def __init__(cls, name, bases, namespace):
        if Struct in bases:
            assert '__fields__' in namespace
            slots = []
            format = '<'
            for datatype, field in namespace['__fields__']:
                slots.append(field)
                if issubclass(datatype, (Struct, Array)):
                    format += '%ds' % datatype.__size__
                else:
                    format += datatype.__format__[1:]
            cls.__slots__ = tuple(slots)
            cls.__format__ = format
        else:
            assert '__format__' in namespace
            cls.__slots__ = ()
        # Update (cache) the record size
        cls.__size__ = struct.calcsize(cls.__format__)
        return super(DataType, cls).__init__(name, bases, namespace)

# allow the DataType metaclass to do its thing on the 'Struct' class
Struct = None
class Struct(object):
    __metaclass__ = DataType
    __format__ = ''

    def __init__(self, bytes=None):
        if bytes is None:
            bytes = '\0' * self.__size__
        values = struct.unpack(self.__format__, bytes)
        for (datatype, name), value in zip(self.__fields__, values):
            setattr(self, name, datatype(value))
        return

    def __repr__(self):
        values = []
        for format, name in self.__fields__:
            value = getattr(self, name)
            if issubclass(format, Struct):
                members = repr(value).split('\n')
                members.insert(0, '<%s>' % format.__name__)
                value = '\n    '.join(members)
            values.append('%s: %s' % (name, value))
        return '\n'.join(values)

    #@classmethod
    def load(cls, stream):
        return cls(stream.read(cls.__size__))
    load = classmethod(load)

    def pack(self):
        values = []
        for datatype, field in self.__fields__:
            value = getattr(self, field)
            if issubclass(datatype, (Struct, Array)):
                value = value.pack()
            values.append(value)
        return struct.pack(self.__format__, *values)

class Array(list):
    def __init__(self, bytes=None):
        if bytes is None:
            bytes = '\0' * self.__size__
        values = struct.unpack(self.__format__, bytes)
        if self.__itemtype__:
            values = map(self.__itemtype__, values)
        list.__init__(self, values)

    def pack(self):
        values = self
        if self.__itemtype__ is not None:
            values = [ item.pack() for item in values ]
        return struct.pack(self.__format__, *values)

class Byte(int):
    __metaclass__ = DataType
    __format__ = '<B'

class Word(int):
    __metaclass__ = DataType
    __format__ = '<H'

class Dword(long):
    __metaclass__ = DataType
    __format__ = '<L'

def ArrayOf(datatype, size):
    name = '%s[%d]' % (datatype.__name__, size)
    if issubclass(datatype, (Struct, Array)):
        format = '%ds' % datatype.__size__
        itemtype = datatype
    else:
        format = datatype.__format__[1:]
        itemtype = None
    namespace = {
        '__metaclass__' : DataType,
        '__format__' : '<' + (format * size),
        '__itemtype__' : itemtype,
        }
    return DataType(name, (Array,), namespace)

IMAGE_NT_OPTIONAL_HDR32_MAGIC = 0x010B
IMAGE_NUMBEROF_DIRECTORY_ENTRIES = 16
IMAGE_SIZEOF_SHORT_NAME = 8
IMAGE_DIRECTORY_ENTRY_RESOURCE = 2

IMAGE_SUBSYSTEM_WINDOWS_GUI = 2
IMAGE_SUBSYSTEM_WINDOWS_CUI = 3

# Predefined Resource Types
RT_RCDATA = 10
RT_VERSION = 16

class ImageDosHeader(Struct):
    __fields__ = [
        (Word, 'e_magic'),
        (Word, 'e_cblp'),
        (Word, 'e_cp'),
        (Word, 'e_crlc'),
        (Word, 'e_cparhdr'),
        (Word, 'e_minalloc'),
        (Word, 'e_maxalloc'),
        (Word, 'e_ss'),
        (Word, 'e_sp'),
        (Word, 'e_csnum'),
        (Word, 'e_ip'),
        (Word, 'e_cs'),
        (Word, 'e_lfarlc'),
        (Word, 'e_ovno'),
        (ArrayOf(Word, 4), 'e_res'),
        (Word, 'e_oemid'),
        (Word, 'e_oeminfo'),
        (ArrayOf(Word, 10), 'e_res2'),
        (Dword, 'e_lfanew'),
        ]

class ImageFileHeader(Struct):
    __fields__ = [
        (Word, 'Machine'),
        (Word, 'NumberOfSections'),
        (Dword, 'TimeDateStamp'),
        (Dword, 'PointerToSymbolTable'),
        (Dword, 'NumberOfSymbols'),
        (Word, 'SizeOfOptionalHeader'),
        (Word, 'Characteristics'),
        ]

class ImageDataDirectory(Struct):
    __fields__ = [
        (Dword, 'VirtualAddress'),
        (Dword, 'Size'),
        ]

class ImageOptionalHeader(Struct):
    __fields__ = [
        (Word, 'Magic'),
        (Byte, 'MajorLinkerVersion'),
        (Byte, 'MinorLinkerVersion'),
        (Dword, 'SizeOfCode'),
        (Dword, 'SizeOfInitializedData'),
        (Dword, 'SizeOfUninitializedData'),
        (Dword, 'AddressOfEntryPoint'),
        (Dword, 'BaseOfCode'),
        (Dword, 'BaseOfData'),
        (Dword, 'ImageBase'),
        (Dword, 'SectionAlignment'),
        (Dword, 'FileAlignment'),
        (Word, 'MajorOperatingSystemVersion'),
        (Word, 'MinorOperatingSystemVersion'),
        (Word, 'MajorImageVersion'),
        (Word, 'MinorImageVersion'),
        (Word, 'MajorSubsystemVersion'),
        (Word, 'MinorSubsystemVersion'),
        (Dword, 'Win32VersionValue'),
        (Dword, 'SizeOfImage'),
        (Dword, 'SizeOfHeaders'),
        (Dword, 'CheckSum'),
        (Word, 'Subsystem'),
        (Word, 'DllCharacteristics'),
        (Dword, 'SizeOfStackReserve'),
        (Dword, 'SizeOfStackCommit'),
        (Dword, 'SizeOfHeapReserve'),
        (Dword, 'SizeOfHepCommit'),
        (Dword, 'LoaderFlags'),
        (Dword, 'NumberOfRvaAndSizes'),
        (ArrayOf(ImageDataDirectory, IMAGE_NUMBEROF_DIRECTORY_ENTRIES),
         'DataDirectory'),
        ]

class ImageNTHeaders(Struct):
    __fields__ = [
        (Dword, 'Signature'),
        (ImageFileHeader, 'FileHeader'),
        (ImageOptionalHeader, 'OptionalHeader'),
        ]

class ImageSectionHeader(Struct):
    __fields__ = [
        (ArrayOf(Byte, IMAGE_SIZEOF_SHORT_NAME), 'Name'),
        (Dword, 'Misc'),
        (Dword, 'VirtualAddress'),
        (Dword, 'SizeOfRawData'),
        (Dword, 'PointerToRawData'),
        (Dword, 'PointerToRelocations'),
        (Dword, 'PointerToLinenumbers'),
        (Word, 'NumberOfRelocations'),
        (Word, 'NumberOfLinenumbers'),
        (Dword, 'Characteristics'),
        ]

class ImageResourceDirectory(Struct):
    __fields__ = [
        (Dword, 'Characteristics'),
        (Dword, 'TimeDateStamp'),
        (Word, 'MajorVersion'),
        (Word, 'MinorVersion'),
        (Word, 'NumberOfNamedEntries'),
        (Word, 'NumberOfIdEntries'),
        ]

class ImageResourceDirectoryEntry(Struct):
    __fields__ = [
        (Dword, 'Id'),
        (Dword, 'Offset'),
        ]

class ImageResourceDataEntry(Struct):
    __fields__ = [
        (Dword, 'OffsetToData'),
        (Dword, 'Size'),
        (Dword, 'CodePage'),
        (Dword, 'Reserved'),
        ]

def FindImageHeaders(image):
    # Read DOS header
    image.seek(0)
    dos_header = ImageDosHeader.load(image)
    if dos_header.e_magic != 0x5A4D: # "MZ"
        raise ValueError("not an EXE file")
    if dos_header.e_lfanew == 0:
        raise ValueError("not a PE file")
    # Read PE header and optional header
    image.seek(dos_header.e_lfanew)
    nt_headers = ImageNTHeaders.load(image)
    if nt_headers.Signature != 0x00004550: # "PE\0\0"
        raise ValueError("invalid PE signature")
    file_header = nt_headers.FileHeader
    if file_header.SizeOfOptionalHeader != ImageOptionalHeader.__size__:
        raise ValueError("invalid header size")
    opt_header = nt_headers.OptionalHeader
    if opt_header.Magic != IMAGE_NT_OPTIONAL_HDR32_MAGIC:
        raise ValueError("invaid header magic");
    return dos_header, nt_headers

def FindResourceSection(image):
    # Scan section headers for resource section
    dos_header, nt_headers = FindImageHeaders(image)
    data_directory = nt_headers.OptionalHeader.DataDirectory
    directory_entry = data_directory[IMAGE_DIRECTORY_ENTRY_RESOURCE]
    if directory_entry.VirtualAddress == 0 or directory_entry.Size == 0:
        raise ValueError("no resources")
    address = directory_entry.VirtualAddress
    for i in xrange(nt_headers.FileHeader.NumberOfSections):
        section = ImageSectionHeader.load(image)
        if address == section.VirtualAddress and section.SizeOfRawData != 0:
            return section
    raise ValueError("no resources")

def FindResourceOffset(image, anyId, resourceId, findSubdir):
    directory = ImageResourceDirectory.load(image)
    # Skip over named entries
    for i in xrange(directory.NumberOfNamedEntries):
        entry = ImageResourceDirectoryEntry.load(image)
    # Now process ID entries
    for i in xrange(directory.NumberOfIdEntries):
        entry = ImageResourceDirectoryEntry.load(image)
        if ((anyId or entry.Id == resourceId) and
            ((entry.Offset & 0x80000000L != 0) == findSubdir)):
            return (entry.Offset & 0x7FFFFFFFL)
    raise ValueError("resource not found")

def SeekToResource(image, resourceType, resourceId):
    """Sets stream `f` to the location of the resource source"""
    section = FindResourceSection(image)

    # Scan the resource directory
    image.seek(section.PointerToRawData)
    offset = FindResourceOffset(image, False, resourceType, True)
    image.seek(section.PointerToRawData + offset)
    offset = FindResourceOffset(image, False, resourceId, True)
    image.seek(section.PointerToRawData + offset)
    offset = FindResourceOffset(image, True, 0, False)
    image.seek(section.PointerToRawData + offset)
    entry = ImageResourceDataEntry.load(image)

    # Sanity check: data_entry.OffsetToData is an RVA.  It's
    # technically possible for the RVA to point to a different section,
    # but we don't support that.
    offset = entry.OffsetToData - section.VirtualAddress
    if offset < 0 or section.SizeOfRawData < (offset + entry.Size):
        raise ValueError("invalid resource")

    # Seek to the resource
    image.seek(section.PointerToRawData + offset)
    return

def UpdateResource(image, resourceType, resourceId, resourceData):
    """
    Replaces the resource in the executable file.
    """
    SeekToResource(image, resourceType, resourceId)
    image.write(resourceData.pack())
    return

def SetSubsystem(image, subsystem):
    """
    Sets the Windows subsystem required to run the executable file.
    """
    if subsystem not in (IMAGE_SUBSYSTEM_WINDOWS_GUI,
                         IMAGE_SUBSYSTEM_WINDOWS_CUI):
        raise ValueError("invalid executable subsystem")

    dos_header, nt_headers = FindImageHeaders(image)
    nt_headers.OptionalHeader.Subsystem = subsystem
    image.seek(dos_header.e_lfanew)
    image.write(nt_headers.pack())
    return



def Test(tester):

    tester.startGroup('Boolean Expressions')

    tester.startTest('Creating test environment')
    from Ft.Xml.XPath import ParsedExpr
    from Ft.Lib import boolean

    from DummyExpr import boolT, boolF
    from DummyExpr import num3, numN4, num4p5, numNan, numInf
    from DummyExpr import strPi, strText, STR_EGG1, STR_EGG2
    from DummyExpr import EMPTY_NODE_SET, ONE_NODE_SET, TWO_NODE_SET
    from DummyExpr import nodesetNum0, nodesetNum2, nodesetNum4

    tests = {
        ParsedExpr.ParsedRelationalExpr : [
            # (opcode, left, right), where opcode is
            #  0: <, 1: <=, 2: >, 3: >=
            ((0, strPi, numN4), boolean.false),
            ((1, strPi, numN4), boolean.false),
            ((2, strPi, numN4), boolean.true),
            ((3, strPi, strPi), boolean.true),
            ((0, numNan, numN4), boolean.false),
            ((1, numNan, numN4), boolean.false),
            ((2, numNan, numN4), boolean.false),
            ((3, numNan, strPi), boolean.false),
            ((1, nodesetNum2, nodesetNum2), boolean.true),
            ((3, nodesetNum2, nodesetNum2), boolean.true),
            ((1, nodesetNum0, nodesetNum2), boolean.true),
            ((3, nodesetNum0, nodesetNum2), boolean.false),
            ((1, nodesetNum4, nodesetNum2), boolean.false),
            ((3, nodesetNum4, nodesetNum2), boolean.true),
            ],
        ParsedExpr.ParsedEqualityExpr : [
            (('=', strPi, strPi), boolean.true),
            (('=', strPi, strText), boolean.false),
            (('!=', strPi, numN4), boolean.true),
            (('=', numNan, strText), boolean.false),
            (('!=', numNan, numN4), boolean.true),
            (('=', numNan, numNan), boolean.false),
            (('!=', numNan, numNan), boolean.true),
            (('=', EMPTY_NODE_SET, boolT), boolean.false),
            (('!=', EMPTY_NODE_SET, boolT), boolean.false),
            (('=', EMPTY_NODE_SET, boolF), boolean.false),
            (('!=', EMPTY_NODE_SET, boolF), boolean.false),
            (('=', EMPTY_NODE_SET, ONE_NODE_SET), boolean.false),
            (('!=', EMPTY_NODE_SET, ONE_NODE_SET), boolean.false),
            (('=', ONE_NODE_SET, EMPTY_NODE_SET), boolean.false),
            (('!=', ONE_NODE_SET, EMPTY_NODE_SET), boolean.false),
            (('=', ONE_NODE_SET, ONE_NODE_SET), boolean.true),
            (('!=', ONE_NODE_SET, ONE_NODE_SET), boolean.false),
            (('=', STR_EGG1, ONE_NODE_SET), boolean.true),
            (('!=', STR_EGG1, ONE_NODE_SET), boolean.false),
            (('=', STR_EGG2, ONE_NODE_SET), boolean.false),
            (('!=', STR_EGG2, ONE_NODE_SET), boolean.true),
            (('=', STR_EGG1, TWO_NODE_SET), boolean.true),
            # Yeah, non-intuitive, but boolean.true acc to XPath spec 3.4
            (('!=', STR_EGG1, TWO_NODE_SET), boolean.true),
            ],
        ParsedExpr.ParsedAndExpr : [
            ((boolT, boolT), boolean.true),
            ((boolT, boolF), boolean.false),
            ((boolF, boolF), boolean.false),
            ],
        ParsedExpr.ParsedOrExpr : [
            ((boolT, boolF), boolean.true),
            ((boolT, boolT), boolean.true),
            ((boolF, boolF), boolean.false),
            ],
        }

    tester.testDone()

    for exprClass, tests in tests.iteritems():
        tester.startTest(exprClass.__name__)
        for args, expected in tests:
            expr = exprClass(*args)
            result = expr.evaluate(None)
            tester.compare(result, expected, repr(expr))
        tester.testDone()

    tester.groupDone()



########################################################################
# $Header: /var/local/cvsroot/4Suite/test/Xml/Xslt/Exslt/test_dynamic.py,v 1.1.2.6 2006/10/30 22:43:22 mbrown Exp $
"""Tests for EXSLT Dynamic"""

from Xml.Xslt import test_harness

TESTS = []

SOURCE = """<doc><otu id='1'/><abuo id='2'/><ato id='3'/><ano id='4'/></doc>"""

# dyn:evaluate()
def test_evaluate1(tester):
    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="*">
        <num>
          <xsl:value-of select="dyn:evaluate(concat('name(/doc/*[', position(), '])'))"/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num>otu</num><num>abuo</num><num>ato</num><num>ano</num></result>'

    source = test_harness.FileInfo(string=SOURCE)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:evaluate()')
    return

TESTS.append(test_evaluate1)

def test_evaluate2(tester):
    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="*">
        <num>
          <xsl:value-of select="dyn:evaluate(/..)"/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num/><num/><num/><num/></result>'

    source = test_harness.FileInfo(string=SOURCE)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:evaluate() with invalid XPath expr')
    return

TESTS.append(test_evaluate2)


# dyn:map()
def test_map1(tester):
    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:map(*, '@id')">
        <num>
          <xsl:value-of select="."/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num>1</num><num>2</num><num>3</num><num>4</num></result>'

    source = test_harness.FileInfo(string=SOURCE)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:map()')
    return

TESTS.append(test_map1)

SOURCE2 = """<doc><otu>1</otu><abuo>2</abuo><ato>3</ato><ano>4</ano></doc>"""


def test_map2(tester):

    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:map(*, 'string(.)')">
        <num>
          <xsl:copy-of select="."/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num><exsl:string xmlns:exsl="http://exslt.org/common">1</exsl:string></num><num><exsl:string xmlns:exsl="http://exslt.org/common">2</exsl:string></num><num><exsl:string xmlns:exsl="http://exslt.org/common">3</exsl:string></num><num><exsl:string xmlns:exsl="http://exslt.org/common">4</exsl:string></num></result>'

    source = test_harness.FileInfo(string=SOURCE2)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:map() w/string expr')
    return

TESTS.append(test_map2)


def test_map3(tester):

    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:map(*, 'number(.)')">
        <num>
          <xsl:copy-of select="."/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num><exsl:number xmlns:exsl="http://exslt.org/common">1</exsl:number></num><num><exsl:number xmlns:exsl="http://exslt.org/common">2</exsl:number></num><num><exsl:number xmlns:exsl="http://exslt.org/common">3</exsl:number></num><num><exsl:number xmlns:exsl="http://exslt.org/common">4</exsl:number></num></result>'

    source = test_harness.FileInfo(string=SOURCE2)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:map() w/numeric expr')
    return

TESTS.append(test_map3)

def test_map4(tester):

    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:map(*, '. > 2')">
        <num>
          <xsl:copy-of select="."/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><num><exsl:boolean xmlns:exsl="http://exslt.org/common"/></num><num><exsl:boolean xmlns:exsl="http://exslt.org/common"/></num><num><exsl:boolean xmlns:exsl="http://exslt.org/common">true</exsl:boolean></num><num><exsl:boolean xmlns:exsl="http://exslt.org/common">true</exsl:boolean></num></result>'

    source = test_harness.FileInfo(string=SOURCE2)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:map() w/boolean expr')
    return

TESTS.append(test_map4)


def test_map5(tester):

    INVALID_EXPRS = ['', '%', 'not()']

    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:map(*, '%s')">
        <num>
          <xsl:copy-of select="."/>
        </num>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result/>'

    for invalid_expr in INVALID_EXPRS:
        source = test_harness.FileInfo(string=SOURCE2)
        transform = test_harness.FileInfo(string=TRANSFORM % invalid_expr)
        test_harness.XsltTest(tester, source, [transform], EXPECTED,
                              title="dyn:map() w/invalid expr %r" % invalid_expr)
    return

TESTS.append(test_map5)


def test_map6(tester):
    MAP6_SRC = """<?xml version="1.0" encoding="UTF-8"?>
<foo>
  <minutes_to_override>
    <duration>0</duration>
    <duration>15</duration>
    <duration>30</duration>
  </minutes_to_override>
  <comma>,</comma>
</foo>"""

    TRANSFORM = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dynamic="http://exslt.org/dynamic"
  exclude-result-prefixes="dynamic">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <xsl:variable name="data" select="/foo/minutes_to_override/duration"/>
    <r><xsl:value-of select="dynamic:map($data, 'concat(., substring( /foo/comma, 1 div (position()!=last()) ))')"/></r>
  </xsl:template>

</xsl:stylesheet>"""

    EXPECTED = """<?xml version="1.0" encoding="UTF-8"?>
<r>0,15,30</r>"""

    source = test_harness.FileInfo(string=MAP6_SRC)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title="dyn:map() w/var and nested func")
    return

TESTS.append(test_map6)


# dyn:closure()
#FIXME: Not a very good test: exercises closure logic, but not dynamic expression evaluation
def test_closure(tester):
    TRANSFORM = """<?xml version='1.0' encoding='UTF-8'?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dyn="http://exslt.org/dynamic"
  exclude-result-prefixes="dyn"
  version="1.0"
>

  <xsl:template match="doc">
    <result>
      <xsl:for-each select="dyn:closure(*, '*[@x]')">
        <node>
          <xsl:value-of select="@x"/>
        </node>
      </xsl:for-each>
    </result>
  </xsl:template>

</xsl:transform>
"""

    SOURCE2 = """<doc><a x='1'><e x='2'/></a><b x='3'><f/></b><c><g x='4'/></c><d x='5'><h x='6'/></d></doc>"""

    EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<result><node>2</node><node>4</node><node>6</node></result>'

    source = test_harness.FileInfo(string=SOURCE2)
    transform = test_harness.FileInfo(string=TRANSFORM)
    test_harness.XsltTest(tester, source, [transform], EXPECTED,
                          title='dyn:closure()')
    return

TESTS.append(test_closure)
    

def Test(tester):
    tester.startGroup('EXSLT Dynamic module')
    for test in TESTS:
        test(tester)
    tester.groupDone()
    return




import cStringIO
from Ft.Xml.Domlette import Print, CanonicalPrint
from Ft.Xml import Parse
from cStringIO import StringIO
import sha, base64

WSA200403_NS = 'http://schemas.xmlsoap.org/ws/2004/03/addressing'


#Note: c14n spec is at: http://www.w3.org/TR/xml-c14n
#exclusive c14n spec is at: http://www.w3.org/TR/xml-exc-c14n/

C14N_EXCL1_DIGEST =  "xSOXT+dlQwo5uT9PbK08of6W9PM="
C14N_EXCL1 = """<wsa:From xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing" ns3:Id="id-7680063" soapenv:mustUnderstand="0"><wsa:Address>http://bosshog.lbl.gov:9999/wsrf/services/SecureCounterService</wsa:Address><wsa:ReferenceProperties><ns1:CounterKey xmlns:ns1="http://counter.com" ns3:Id="10112">10577413</ns1:CounterKey></wsa:ReferenceProperties></wsa:From>"""

C14N_INC1_DIGEST =  "qdU4f7/+BeHV/JlVGIPM90fNeV8="
C14N_INC1 = """<wsa:From xmlns:ns1="http://counter.com" xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ns3:Id="id-7680063" soapenv:mustUnderstand="0"><wsa:Address>http://bosshog.lbl.gov:9999/wsrf/services/SecureCounterService</wsa:Address><wsa:ReferenceProperties><ns1:CounterKey ns3:Id="10112">10577413</ns1:CounterKey></wsa:ReferenceProperties></wsa:From>"""

C14N_EXCL2_DIGEST = "+IEqF6DRo36Bh93A06S7C4Cmcuo="
C14N_EXCL2 = """<wsa:From xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ns3:Id="id-7680063" soapenv:mustUnderstand="0"><wsa:Address>http://bosshog.lbl.gov:9999/wsrf/services/SecureCounterService</wsa:Address><wsa:ReferenceProperties><ns1:CounterKey xmlns:ns1="http://counter.com" ns3:Id="10112">10577413</ns1:CounterKey></wsa:ReferenceProperties></wsa:From>"""


C14N_EXCL3_DIGEST = "VJvTr+Mx3TeWsQY6iwGbhAJ9/eA="
C14N_EXCL3 = """<soapenv:Body xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="id-28219008"><RequestSecurityTokenResponse xmlns="http://schemas.xmlsoap.org/ws/2004/04/trust"><wsa:EndpointReference xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing"><wsa:Address>http://131.243.2.147:8888/wsrf/services/DelegationService</wsa:Address><wsa:ReferenceProperties><ns1:DelegationKey xmlns:ns1="http://www.globus.org/08/2004/delegationService">8adaa710-ba01-11da-bc99-cbed73daa755</ns1:DelegationKey></wsa:ReferenceProperties></wsa:EndpointReference></RequestSecurityTokenResponse></soapenv:Body>"""


XML_INST1 = """<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><soapenv:Header>
    <wsse:Security soapenv:mustUnderstand="1" xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"><ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
    <ds:SignedInfo>
    <ds:CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="soapenv wsa xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:CanonicalizationMethod>
    <ds:SignatureMethod Algorithm="http://www.globus.org/2002/04/xmlenc#gssapi-sign"/>
    <ds:Reference URI="#id-8409752">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="wsa xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>m9pihAqIBdcdk7ytDvccj89eWi8=</ds:DigestValue>
    </ds:Reference>
    <ds:Reference URI="#id-11434871">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>ofD+Ket5kzR2u/5jWKbFTMtmigk=</ds:DigestValue>
    </ds:Reference>
    <ds:Reference URI="#id-19645447">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>SoQ7RlJa3r94weDWBuWAg/BvydQ=</ds:DigestValue>
    </ds:Reference>
    <ds:Reference URI="#id-5428820">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>z6sCEkkRJrCuY/C0S5b+46WfyMs=</ds:DigestValue>
    </ds:Reference>
    <ds:Reference URI="#id-7680063">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>+IEqF6DRo36Bh93A06S7C4Cmcuo=</ds:DigestValue>
    </ds:Reference>
    <ds:Reference URI="#id-28476580">
    <ds:Transforms>
    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"><ec:InclusiveNamespaces PrefixList="xsd xsi" xmlns:ec="http://www.w3.org/2001/10/xml-exc-c14n#"/></ds:Transform>
    </ds:Transforms>
    <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
    <ds:DigestValue>NFltkKAJpmMkPbJQj5MW1qVceto=</ds:DigestValue>
    </ds:Reference>
    </ds:SignedInfo>
    <ds:SignatureValue>AAAAAAAAAAMAAAvZTrXlZjRSO7tP12tId+lehprEKgk=</ds:SignatureValue>
    <ds:KeyInfo>
    <wsse:SecurityTokenReference><wsse:Reference URI="#SecurityContextToken-32970611" ValueType="http://www.globus.org/ws/2004/09/security/sc#GSSAPI_CONTEXT_TOKEN"/></wsse:SecurityTokenReference>
    </ds:KeyInfo>
    </ds:Signature><wsc:SecurityContextToken wsu:Id="SecurityContextToken-32970611" xmlns:wsc="http://schemas.xmlsoap.org/ws/2004/04/sc" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"><wsc:Identifier>3b1ef410-ab3d-11da-9436-88b687faed94</wsc:Identifier></wsc:SecurityContextToken></wsse:Security><wsa:MessageID wsu:Id="id-11434871" soapenv:mustUnderstand="0" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">uuid:3d592ca0-ab3d-11da-9436-88b687faed94</wsa:MessageID><wsa:To wsu:Id="id-19645447" soapenv:mustUnderstand="0" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">http://schemas.xmlsoap.org/ws/2004/03/addressing/role/anonymous</wsa:To><wsa:Action wsu:Id="id-5428820" soapenv:mustUnderstand="0" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">http://counter.com/CounterPortType/addResponse</wsa:Action><wsa:From ns3:Id="id-7680063" soapenv:mustUnderstand="0" xmlns:ns1="http://counter.com" xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"><wsa:Address>http://bosshog.lbl.gov:9999/wsrf/services/SecureCounterService</wsa:Address><wsa:ReferenceProperties><ns1:CounterKey ns3:Id="10112">10577413</ns1:CounterKey></wsa:ReferenceProperties></wsa:From><wsa:RelatesTo RelationshipType="wsa:Reply" wsu:Id="id-28476580" soapenv:mustUnderstand="0" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">uuid:1141449047.05</wsa:RelatesTo></soapenv:Header><soapenv:Body wsu:Id="id-8409752" xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"><addResponse xmlns="http://counter.com">13</addResponse></soapenv:Body></soapenv:Envelope>"""



def Test(tester):
    dom = Parse(XML_INST1)
    el = filter(lambda E:  (E.namespaceURI, E.localName) == (WSA200403_NS, "From"),
                     dom.childNodes[0].childNodes[0].childNodes)[0]

    tester.startGroup('eclusive c14n (WS-Addressing example)')
    tester.startTest('regular c14n')

    stream = StringIO()
    CanonicalPrint(el, stream, exclusive=False)
    cxml = stream.getvalue().strip()

    d1 = base64.encodestring(sha.sha(C14N_INC1).digest()).strip()
    d2 = base64.encodestring(sha.sha(cxml).digest()).strip()
    #tester.compare(d1, d2)
    #tester.compare(d1, C14N_INC1_DIGEST)
    tester.compare(C14N_INC1, cxml)

    tester.testDone()

    tester.startTest('exclusive c14n')

    stream = StringIO()
    CanonicalPrint(el, stream, exclusive=True)
    cxml = stream.getvalue().strip()

    tester.compare(C14N_EXCL1, cxml)

    tester.testDone()

    tester.startTest('exclusive c14n with inclusivePrefixes')

    stream = StringIO()
    CanonicalPrint(el, stream, exclusive=True, inclusivePrefixes=['xsi', 'xsd'])
    cxml = stream.getvalue().strip()

    tester.compare(C14N_EXCL2, cxml)

    tester.testDone()

    tester.groupDone()

    del dom
    del el

    return

